==> waku-0.8.0.tar.gz: waku-0.8.0/tests/test_application.py <==
from __future__ import annotations

from contextlib import asynccontextmanager
from typing import TYPE_CHECKING

from dishka import Scope

from waku import WakuApplication, WakuFactory
from waku.di import provide
from waku.modules import module

if TYPE_CHECKING:
    from collections.abc import AsyncIterator


class MockLifespanManager:
    def __init__(self) -> None:
        self.entered = False
        self.exited = False

    @asynccontextmanager
    async def __call__(self, _: WakuApplication) -> AsyncIterator[None]:
        self.entered = True
        yield
        self.exited = True


async def test_application_lifespan() -> None:
    """Test application startup and shutdown lifecycle."""
    manager_1 = MockLifespanManager()
    manager_2 = MockLifespanManager()

    @module()
    class AppModule:
        pass

    application = WakuFactory(
        AppModule,
        lifespan=[manager_1, manager_2],
    ).create()

    async with application:
        assert manager_1.entered
        assert manager_2.entered
        assert not manager_1.exited
        assert not manager_2.exited

    assert manager_1.exited
    assert manager_2.exited  # type: ignore[unreachable]


async def test_application_module_registration() -> None:
    """Test that modules are properly registered with the application."""

    class ServiceA:
        pass

    class ServiceB:
        pass

    @module(providers=[provide(ServiceA)], exports=[ServiceA])
    class ModuleA:
        pass

    @module(providers=[provide(ServiceB, scope=Scope.APP)], imports=[ModuleA])
    class ModuleB:
        pass

    @module(imports=[ModuleA, ModuleB])
    class AppModule:
        pass

    application = WakuFactory(AppModule).create()

    async with application, application.container() as request_container:
        await request_container.get(ServiceA)
        await application.container.get(ServiceB)

==> waku-0.9.0.tar.gz: waku-0.9.0/tests/test_application.py <==
from __future__ import annotations

from contextlib import asynccontextmanager
from typing import TYPE_CHECKING

from waku import WakuApplication, WakuFactory
from waku.di import scoped, singleton
from waku.modules import module

if TYPE_CHECKING:
    from collections.abc import AsyncIterator


class MockLifespanManager:
    def __init__(self) -> None:
        self.entered = False
        self.exited = False

    @asynccontextmanager
    async def __call__(self, _: WakuApplication) -> AsyncIterator[None]:
        self.entered = True
        yield
        self.exited = True


async def test_application_lifespan() -> None:
    """Test application startup and shutdown lifecycle."""
    manager_1 = MockLifespanManager()
    manager_2 = MockLifespanManager()

    @module()
    class AppModule:
        pass

    application = WakuFactory(
        AppModule,
        lifespan=[manager_1, manager_2],
    ).create()

    async with application:
        assert manager_1.entered
        assert manager_2.entered
        assert not manager_1.exited
        assert not manager_2.exited

    assert manager_1.exited
    assert manager_2.exited  # type: ignore[unreachable]


async def test_application_module_registration() -> None:
    """Test that modules are properly registered with the application."""

    class ServiceA:
        pass

    class ServiceB:
        pass

    @module(providers=[scoped(ServiceA)], exports=[ServiceA])
    class ModuleA:
        pass

    @module(providers=[singleton(ServiceB)], imports=[ModuleA])
    class ModuleB:
        pass

    @module(imports=[ModuleA, ModuleB])
    class AppModule:
        pass

    application = WakuFactory(AppModule).create()

    async with application, application.container() as request_container:
        await request_container.get(ServiceA)
        await application.container.get(ServiceB)
